////////////////////////////////////////////////////////////////////////
//
//        File  Name : sokcet.cc
//        Class Name : socket_type
//        Author     : Young-ki Hwang
//        Date       : April 30, 1996
//        Description: This is the methods of "socket_type" class.
//
///////////////////////////////////////////////////////////////////////

#include <iostream.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include "socket.h"

// From the C library functions
extern "C" socket(int, int, int);
extern "C" read(int, char *, int);
extern "C" write(int, char *, int);
extern "C" close(int);
extern "C" setsockopt(int, int, int, int*, int);


//
// constrcutor:socket_type()
// Open the socket with communication protocol desired(Internet TCP,
// Internet UDP, XNS SPP etc)
//
socket_type::socket_type()
{  int val =1;

   // open a socket and return the socket descriptor
   if( ( sockfd = socket(AF_INET, SOCK_STREAM, 0) ) < 0 )
      { cout<< "Error : can't open stream socket.";
        exit(1);
      }

   // set option for the socket
   if( setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val)) < 0 )
     { cout<< "Error : Socket option error.";
       exit(1);
     }
}


//
//  receive()
//  Receive a message from the socket which is opened in the constructor.
//  If it succeed, return 1, otherwise return 0. 
//
int socket_type::receive(char *buffer)
{  int    nread, len;

   // receive a message into the buffer with message size
   if( (nread = read(sockfd, buffer, MAX_MSG_SIZE)) < 0 )
     { if(errno == EWOULDBLOCK) return(0);
       else
         { cout<< "Error :  can't read data from stream socket.";
           exit(1);
	 }
     }
   else
     { buffer[nread]='\0';
       return(1);
     }
}


//
//  receive()
//  Receive a message from the socket which is from the argument.
//  If it succeed, return 1, otherwise return 0. 
//
int socket_type::receive(int socketfd, char *buffer)
{  int   nread, len;
  
 
   // receive a message into the buffer with message size
   if( (nread = read(socketfd, buffer, MAX_MSG_SIZE)) < 0 )
     { if(errno == EWOULDBLOCK) return(0);
       else
         { cout<< "Error :  can't read data from stream socket.";
           exit(1);
	 }
     }
   else
     { buffer[nread]='\0';
       return(1);
     }
}


//
//  send()
//  Send a message into the socket which is opened in the constructor
//  If it succeed, return 1, otherwise return 0. 
//
int socket_type::send(char *buffer)
{  int  nread, length;
   char temp;

   length = string1.get_length(buffer);
   
   // send a message in the buffer into the socket.
   if( (nread = write(sockfd, buffer, MAX_MSG_SIZE)) < 0 )
     { if(errno == EWOULDBLOCK) return(0);
       else
         { cout<< "Error :  can't write data to stream socket.";
	   exit(1);
	 }
     }
   else return(1);
}



//
//  send()
//  send a message into the socket which is from the argument.
//  If it succeed, return 1, otherwise return 0. 
//
int socket_type::send(int socketfd, char *buffer)
{  int  nread, length;
   char temp;

   length = string1.get_length(buffer);

   // send a message in the buffer into the socket.
   if( (nread = write(socketfd, buffer, MAX_MSG_SIZE)) < 0 )
     { if(errno == EWOULDBLOCK) return(0);
       else
         { cout<< "Error :  can't write data to stream socket.";
	   exit(1);
	 }
     }
   else return(1);
}



//
//  get_socketfd()
//  get a socket file descriptor which is opened in the constructor.
//
int socket_type::get_socketfd()   
{
   return(sockfd);
}



//
//  close_sokcet()
//  close a sokcet file descriptor
//
socket_type::close_socket()
{ 
  close(sockfd);
}



//
// destructor:~socket_type()
// Close a socket file descriptor which is opened in the constructor
//
socket_type::~socket_type()
{
   close(sockfd);
}



